package service

import (
	"context"

	def "git.pbiernat.dev/egommerce/basket-service/internal/app/definition"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/event"
	"git.pbiernat.dev/egommerce/go-api-pkg/fluentd"
	amqp "git.pbiernat.dev/egommerce/go-api-pkg/rabbitmq"
	"github.com/georgysavva/scany/v2/pgxscan"
	"github.com/jackc/pgx/v5/pgxpool"
	base "github.com/streadway/amqp"
)

type BasketService struct {
	dbConn *pgxpool.Pool
	ebCh   *base.Channel
	log    *fluentd.Logger
}

func NewBasketService(dbConn *pgxpool.Pool, chn *base.Channel, log *fluentd.Logger) *BasketService {
	return &BasketService{dbConn, chn, log}
}

func (s *BasketService) Create(basketID string) (*def.BasketModel, error) {
	ctx := context.Background()

	sql := `INSERT INTO "basket-svc".basket(id) VALUES($1)`
	if _, err := s.dbConn.Exec(ctx, sql, basketID); err != nil {
		return nil, err
	}

	return &def.BasketModel{ID: basketID}, nil // FIXME - WTF is that xD??
}

func (s *BasketService) FetchFromDB(basketID string) (*def.BasketModel, error) {
	ctx := context.Background()

	basket := new(def.BasketModel)
	err := pgxscan.Get(ctx, s.dbConn, basket, `SELECT id, created_at, updated_at FROM "basket-svc".basket WHERE id=$1`, basketID)
	if err != nil {
		return nil, err
	}

	return basket, nil
}

func (s *BasketService) AddProduct(productId, basketID string, qty int) error {
	ctx := context.Background()
	s.log.Log("Adding product#:%s into Basket#:%s", productId, basketID)

	sql := `INSERT INTO "basket-svc".basket_item(basket_id, product_id) VALUES($1,$2)`
	if _, err := s.dbConn.Exec(ctx, sql, basketID, productId); err != nil {
		return err
	}

	return nil
}

func (s *BasketService) Checkout(reqID, basketID string) (string, error) {
	s.log.Log("Creating initial order from basket#:%s", basketID)

	msg := &event.BasketCheckoutEvent{Event: event.NewEvent(reqID), BasketID: basketID}
	amqp.Publish(s.ebCh, "api-events", "basket.order.basketCheckout", msg)

	return basketID, nil
}
